package ru.ezsk.swing;

import java.awt.GridBagConstraints;

/**
 * @version $Revision: 1.13 $ $Date: 2004/07/26 15:58:19 $ $Author: beholder $
 */
public class GBC extends GridBagConstraints
{
    //========================================================= Nested Classes

    public static enum Anchor
    {
        center          (GridBagConstraints.CENTER),
        north           (GridBagConstraints.NORTH),
        northEast       (GridBagConstraints.NORTHEAST),
        east            (GridBagConstraints.EAST),
        southEast       (GridBagConstraints.SOUTHEAST),
        south           (GridBagConstraints.SOUTH),
        southWest       (GridBagConstraints.SOUTHWEST),
        west            (GridBagConstraints.WEST),
        northWest       (GridBagConstraints.NORTHWEST),
        pageStart       (GridBagConstraints.PAGE_START),
        pageEnd         (GridBagConstraints.PAGE_END),
        lineStart       (GridBagConstraints.LINE_START),
        lineEnd         (GridBagConstraints.LINE_END),
        firstLineStart  (GridBagConstraints.FIRST_LINE_START),
        firstLineEnd    (GridBagConstraints.FIRST_LINE_END),
        lastLineStart   (GridBagConstraints.LAST_LINE_START),
        lastLineEnd     (GridBagConstraints.LAST_LINE_END);

        final int value;

        private Anchor(int value) {
            this.value = value;
        }
    }

    public static enum Fill
    {
        none        (GridBagConstraints.NONE),
        horizontal  (GridBagConstraints.HORIZONTAL),
        vertical    (GridBagConstraints.VERTICAL),
        both        (GridBagConstraints.BOTH);

        final int value;

        private Fill(int value) {
            this.value = value;
        }
    }

    //=========================================================== Constructors

    public GBC() {
    }

    public GBC(int x, int y) {
        gridx = x;
        gridy = y;
    }

    public GBC(int x, int y, int colSpan, int rowSpan) {
        gridx = x;
        gridy = y;
        gridwidth  = colSpan;
        gridheight = rowSpan;
    }

    //================================================================ Methods

    public GBC reset() {
        gridx = RELATIVE;
        gridy = RELATIVE;
        gridwidth  = 1;
        gridheight = 1;

        weightx = 0;
        weighty = 0;
        anchor = CENTER;
        fill = NONE;

        insets.set(0, 0, 0, 0);
        ipadx = 0;
        ipady = 0;

        return this;
    }

    public GBC xy(int x, int y) {
        gridx = x;
        gridy = y;
        return this;
    }

    public GBC relative() {
        gridx = RELATIVE;
        gridy = RELATIVE;
        return this;
    }

    public GBC nextX() {
        final int shift = (gridwidth > 0) ? gridwidth : 1;
        gridx += shift;
        return this;
    }

    public GBC nextY() {
        final int shift = (gridheight > 0) ? gridheight : 1;
        gridy += shift;
        return this;
    }

    public GBC newLine() {
        nextY();
        gridx = 0;
        return this;
    }

    public GBC span(int col, int row) {
        gridwidth  = col;
        gridheight = row;
        return this;
    }

    public GBC nospan() {
        gridheight = 1;
        gridwidth  = 1;
        return this;
    }

    public GBC xySpan(int x, int y, int col, int row) {
        gridx = x;
        gridy = y;
        gridwidth  = col;
        gridheight = row;
        return this;
    }

    /**
     * Sets whether to resize the component, and if so, how.
     *
     * @param fillMode the following values are valid: <ul>
     *      <li>{@link #NONE} : Do not resize the component.</li>
     *      <li>{@link #HORIZONTAL} : Make the component wide enough to fill
     *          its display area horizontally, but do not change its
     *          height.</li>
     *      <li>{@link #VERTICAL} : Make the component tall enough to fill its
     *          display area vertically, but do not change its width.</li>
     *      <li>{@link #BOTH} : Make the component fill its display area
     *         entirely.</li>
     *      </ul>
     *
     * @return this object.
     */
    public GBC fill(Fill fillMode) {
        fill = fillMode.value;
        return this;
    }

    public GBC pad(int xpad, int ypad) {
        ipadx = xpad;
        ipady = ypad;
        return this;
    }

    public GBC pad(int pad) {
        ipadx = pad;
        ipady = pad;
        return this;
    }

    public GBC margin(int top, int left, int bottom, int right) {
        insets.set(top, left, bottom, right);
        return this;
    }

    public GBC margin(int value) {
        return margin(value, value, value, value);
    }

    /**
     * Sets where, within the display area, to place the component.
     *
     * @param anAnchor there are two kinds of possible values: relative and
     *      absolute.  Relative values are interpreted relative to the
     *      container's component orientation property while absolute values
     *      are not.
     *      <p>The absolute values are:
     *      {@link #CENTER}, {@link #NORTH}, {@link #NORTHEAST},
     *      {@link #EAST}, {@link #SOUTHEAST}, {@link #SOUTH},
     *      {@link #SOUTHWEST}, {@link #WEST}, and
     *      {@link #NORTHWEST}.
     *      <p>The relative values are: {@link #PAGE_START},
     *      {@link #PAGE_END},
     *      {@link #LINE_START}, {@link #LINE_END},
     *      {@link #FIRST_LINE_START}, {@link #FIRST_LINE_END},
     *      {@link #LAST_LINE_START} and {@link #LAST_LINE_END}.
     *      <p>The default value is {@link #CENTER}.
     *
     * @return this object.
     */
    public GBC anchor(Anchor anAnchor) {
        anchor = anAnchor.value;
        return this;
    }

    public GBC center() {
        anchor = CENTER;
        return this;
    }

    public GBC weight(double wx, double wy) {
        weightx = wx;
        weighty = wy;
        return this;
    }

    @Override public GBC clone() {
        return (GBC) super.clone();
    }
}
